;; (c) 2008 duane a. bailey
;; This file contains all objects that are instantiated before the start
;; of the Mead system.
(module Builtins mzscheme
  (require (lib "Syntax.ss" "Mead")
           (lib "Math.ss" "Mead")
           (lib "Object.ss" "Mead")
	   (lib "SimpleObjects.ss" "Mead")
           (lib "Lights.ss" "Mead")
           (lib "Cameras.ss" "Mead")
           )
  (provide meadVersion meadCopyright
	   black dkGray gray ltGray white
           red green blue cyan magenta yellow
           blackPlaster dkGrayPlaster grayPlaster ltGrayPlaster whitePlaster
           redPlaster greenPlaster bluePlaster cyanPlaster magentaPlaster yellowPlaster
           blackPlastic dkGrayPlastic grayPlastic ltGrayPlastic whitePlastic
           redPlastic greenPlastic bluePlastic cyanPlastic magentaPlastic yellowPlastic

	   mirrorMat

	   identity
           theContext bulb scene image camera
	   cube sphere cone cylinder greenGlass
           )

; the following line is edited during the package building process
; copyright information
(define (meadVersion) "Mead 1.85")
(define (meadCopyright) "Mead 1.85, (c) 2007,2008 Duane A. Bailey")

; basic colors, in RGB format.
(define black '(0 0 0))
(define dkGray '(0.25 0.25 0.25))
(define gray '(0.5 0.5 0.5))
(define ltGray '(0.75 0.75 0.75))
(define white '(1 1 1))
(define red '(1 0 0))
(define green '(0 1 0))
(define blue '(0 0 1))
(define cyan '(0 1 1))
(define magenta '(1 0 1))
(define yellow '(1 1 0))

; "plaster" materials associated with each of the basic colors
(object blackPlaster Material
        (type 'plaster)
        (color black))
(object dkGrayPlaster Material 
        (type 'plaster) 
        (color dkGray))
(object grayPlaster Material 
        (type 'plaster) 
        (color gray))
(object ltGrayPlaster Material 
        (type 'plaster) 
        (color ltGray))
(object whitePlaster Material 
        (type 'plaster) 
        (color white))
(object redPlaster Material
        (type 'plaster) 
        (color red))
(object yellowPlaster Material 
        (type 'plaster) 
        (color yellow))
(object greenPlaster Material 
        (type 'plaster) 
        (color green))
(object cyanPlaster Material 
        (type 'plaster) 
        (color cyan))
(object bluePlaster Material 
        (type 'plaster) 
        (color blue))
(object magentaPlaster Material 
        (type 'plaster) 
        (color magenta))

; "plastic" materials associated with each of the basic colors
(object blackPlastic Material
        (type 'plastic)
        (color black))
(object dkGrayPlastic Material 
        (type 'plastic) 
        (color dkGray))
(object grayPlastic Material 
        (type 'plastic) 
        (color gray))
(object ltGrayPlastic Material 
        (type 'plastic) 
        (color ltGray))
(object whitePlastic Material 
        (type 'plastic) 
        (color white))
(object redPlastic Material
        (type 'plastic) 
        (color red))
(object yellowPlastic Material 
        (type 'plastic) 
        (color yellow))
(object greenPlastic Material 
        (type 'plastic) 
        (color green))
(object cyanPlastic Material 
        (type 'plastic) 
        (color cyan))
(object bluePlastic Material 
        (type 'plastic) 
        (color blue))
(object magentaPlastic Material 
        (type 'plastic) 
        (color magenta))

; a very reflective material for use in mirrors
(object mirrorMat Material
	(type 'mirror)
	(color gray)
)

; a poor Coke-bottle glass material
(object greenGlass Material
	(type 'glass)
	(color green)
)

; each of these objects is an example of its respective class
(object cube Cube)
(object sphere Sphere)
(object cylinder Cylinder)
(object cone Cone)

; the main context - describing the transform stacks, etc.
(define theContext (new Context))

; the name of non-acting transformation
(object identity Transform)

; a white light bulb
(object bulb Light 
	(description "A white light bulb.")
        (color white))

; the main scene, where everything is constructed
(object scene Group
	(description "The main scene.")
        (add bulb (translate 0 300 -300))
        )

; the default camera, used to shoot standard images
(object camera Camera
	(description "The default camera.")
        (pos '(0 0 -500))
        (coi '(0 0 0))
        (subject 'scene)
        )

; the image description used by the default camera
(define image (tell camera (getImage)))
(tell image
      (description "The image associated with the default camera.")
      (background cyan)
)
)
