(module CSG mzscheme
  (require (lib "Syntax.ss" "Mead")
           (lib "Math.ss" "Mead")
           (lib "Object.ss" "Mead"))
  (provide Union Intersection Difference
           csg+ csg- csg* csg+=)
  
(define Union
  (class Group 
    (inherit description attr getAttr getXform material getMaterial getTags forward)
    (define/public (merge)
      (attr 'merge #t))
    (define/public (merge?)
      (getAttr 'merge))
    
    (define/override (POVDescribe context)
      (let* ([refs (getAttr 'references)]
             [t (getXform)]
             [m (getMaterial)]
             [g (getTags)]
	     [vg (tell context (getVerboseGrouping))])
        (begin
	  (tell context (verboseGrouping #t) (pushMaterial m) (pushTransform t) (pushTags g))
	  (if (merge?)
	      (printf "merge {\n")
	      (printf "union {\n")
	      )
	  (for-each (lambda (o) (tell1 (cadr o) POVDescribe context)) refs)
	  (printf "}\n")
	  (tell context (verboseGrouping vg) (popTransform) (popMaterial) (popTags)))))

    ; object construction:
    (begin
      (super-new)
      (description "A Mead CSG union")
      )
    )
  )

(define Intersection
  (class Group 
    (inherit description attr getAttr getXform material getMaterial getTags forward)
    
    (define/override (POVDescribe context)
      (let* ([refs (getAttr 'references)]
             [t (getXform)]
             [m (getMaterial)]
             [g (getTags)]
	     [vg (tell context (getVerboseGrouping))])
        (begin
          (tell context (verboseGrouping #t) (pushMaterial m) (pushTransform t) (pushTags g))
          (printf "intersection {\n")
          (for-each (lambda (o) (tell1 (cadr o) POVDescribe context)) refs)
          (printf "}\n")
          (tell context (verboseGrouping vg) (popTransform) (popMaterial) (popTags)))))
    
    ; object construction:
    (begin
      (super-new)
      (description "A Mead CSG intersection")
      )
    )
  )

(define Difference
  (class Group 
    (inherit description attr getAttr getXform material getMaterial getTags forward)
    
    (define/override (POVDescribe context)
      (let* ([refs (getAttr 'references)]
             [t (getXform)]
             [m (getMaterial)]
             [g (getTags)]
	     [vg (tell context (getVerboseGrouping))])
        (begin
          (tell context (verboseGrouping #t) (pushMaterial m) (pushTransform t) (pushTags g))
          (printf "difference {\n")
          (for-each (lambda (o) (tell1 (cadr o) POVDescribe context)) refs)
          (printf "}\n")
          (tell context (verboseGrouping vg) (popTransform) (popMaterial) (popTags)))))
    
    ; object construction:
    (begin
      (super-new)
      (description "A Mead CSG difference")
      )
    )
  )

(define-syntax csg+
  (syntax-rules ()
    ([csg+ (o . msgs) ...]
     (let ([tt (new Union)])
       (tell tt (add o . msgs))
       ...
       tt))))

(define-syntax csg+=
  (syntax-rules ()
    ([csg+ (o . msgs) ...]
     (let ([tt (new Union)])
       (tell1 tt merge)
       (tell tt (add o . msgs))
       ...
       tt))))

(define-syntax csg-
  (syntax-rules ()
    ([csg- (o . msgs) ...]
     (let ([tt (new Difference)])
       (tell tt (add o . msgs))
       ...
       tt))))

(define-syntax csg*
  (syntax-rules ()
    ([csg* (o . msgs) ...]
     (let ([tt (new Intersection)])
       (tell tt (add o . msgs))
       ...
       tt))))
)
