(module Cameras mzscheme
  (require (lib "process.ss")
           (lib "Syntax.ss" "Mead")
           (lib "Math.ss" "Mead")
           (lib "Object.ss" "Mead")) ;; this incorporates Materials, as well.
  (provide POVWriter pov
           Image Camera StereoCamera IsometricCamera)
(define POVWriter
  (class object%
    (define/public (writePoint p)
      (let* ([x (car p)]
             [y (cadr p)]
             [z (caddr p)])
        (printf "<~a, ~a, ~a>" x y z)))
    
    (define/public (writeColor c)
      (let ((l (length c)))
        (cond ([= l 0] (printf "color rgb <0.5,0.5,0.5>"))
              ([<= l 2] (printf "color rgb <~a,~a,~a>" (car c) (car c) (car c)))
              ([= l 3] (printf "color rgb <~a,~a,~a>" (car c) (cadr c) (caddr c)))
              ([= l 4] (printf "color rgbf <~a,~a,~a,~a>" (car c) (cadr c) (caddr c) (- 1 (cadddr c))))
              ([>= l 5] (printf "color <~a,~a,~a,~a,~a>" (car c) (cadr c) (caddr c) (- 1 (cadddr c)) (cadr (cdddr c)))))))
    
    (begin
      (super-new)
      )))
(object pov POVWriter)

(define Image
  (class MeadObject
    (inherit attr getAttr description)
    (public frameNumber getFrameNumber
	    fileName getFileName getFilePath
	    getFileNameSwitches
	    height getHeight
	    viewResult getViewResult
	    keep getKeep debug getDebug
            width getWidth
            dimensions getDimensions
            aspectRatio getAspectRatio
            quality getQuality
            antiAlias getAntiAlias
            background getBackground
	    fog groundFog getFog
	    fogColor getFogColor
	    fogAttenuation getFogAttenuation
            POVShoot POVProof POVDescribe  advanceFrame rewindFrame)
    (private setAspectRatio setWidth setHeight)

    (define (viewResult b)
      (attr 'viewResult b))
    (define (getViewResult)
      (getAttr 'viewResult))
    (define (keep b)
      (attr 'keep b))
    (define (getKeep)
      (getAttr 'keep))
    (define (debug b)
      (attr 'debug b))
    (define (getDebug)
      (getAttr 'debug))
    (define (setAspectRatio r)
      (attr 'aspectRatio r))
    (define (aspectRatio r)
      (attr 'aspectRatio r)
      (if (> (getWidth) 0) (setHeight (/ (getWidth) r)))
    )
    (define (getAspectRatio)
      (getAttr 'aspectRatio)
      )
    
    (define (setWidth w)
      (attr 'width w)
      )
    (define (width w)
      (attr 'width w)
      (setHeight (/ w (getAspectRatio)))
      )
    (define (getWidth)
      (getAttr 'width)
      )
    
    (define (setHeight h)
      (attr 'height h)
      )
    (define (height h)
      (width (* (getAspectRatio) h)) ;; ultimately sets height
      )
    (define (getHeight)
      (getAttr 'height))
    
    (define (dimensions w h)
      (setWidth w)
      (setHeight h)
      (setAspectRatio (/ w h))
      )
    (define (getDimensions)
      (list (getWidth) (getHeight)))
    
    (define (quality q)
      (attr 'quality q)
      )
    (define (getQuality)
      (getAttr 'quality)
      )
    
    (define (antiAlias v)
      (attr 'antiAlias v)
      )
    (define (getAntiAlias)
      (getAttr 'antiAlias)
      )

    ;; environmental options
    (define (background c)
      (attr 'background c)
      )
    (define (getBackground)
      (getAttr 'background)
      )
    (define (fog)
      (attr 'fog 1)
      )
    (define (groundFog)
      (attr 'fog 2)
      )
    (define (getFog)
      (getAttr 'fog)
      )
    (define (fogColor c)
      (attr 'fogColor c)
      )
    (define (getFogColor)
      (getAttr 'fogColor)
      )
    (define (fogAttenuation d)
      (attr 'fogAttenuation d)
      )
    (define (getFogAttenuation)
      (getAttr 'fogAttenuation)
      )
    (define (frameNumber n)
      (attr 'frameNumber n))
    (define (getFrameNumber)
      (getAttr 'frameNumber))
    (define (fileName s)
      (attr 'fileName s))
    (define (getFileName)
      (getAttr 'fileName))
    (define (getFilePath)
      (if (not (getFileName)) #f
	(let* ([home (or (getenv "HOME") "/tmp")]
	       [user (or (getenv "USER") "untitled")]
	       [f (getFileName)]
	       [ef (tell environment (getProjectFolder))]
	       [folder (or (and (string? ef) ef) "/tmp/")]
	       [file (or (and (string? f) (< 0 (string-length f)) f) user)])
	  (if (and (< 0 (string-length file)) (eq? #\/ (string-ref file 0)))
	    (begin
	      (set! home "")
	      (set! folder "")))
	  (if (and (< 0 (string-length folder)) (eq? #\/ (string-ref folder 0)))
	    (begin
	      (set! home "")))
	  (string-append home "/" folder "/" file)
         )
	)
      )

    (define (advanceFrame)
      (if (getFrameNumber) (frameNumber (+ 1 (getFrameNumber)))))

    (define (rewindFrame)
      (if (getFrameNumber) (frameNumber (- (getFrameNumber) 1))))

    (define (getFileNameSwitches)
      (let* ([fp (getFilePath)]
	     [n (getFrameNumber)]
	     [ks (if (getKeep) " --keep" "")]
	     [ds (if (getDebug) " --debug" "")]
	     [vsw (if (getViewResult) "" " --batch")]
	     [sw (string-append ks ds vsw)]
	     )
	(if fp
	    (if n
		(begin
		  (advanceFrame)
		  (string-append " --output \"" fp "\" --frame " (format "~a" n) sw)
		  )
		(string-append " --output \"" fp "\"" sw)
		)
	    ""
	    )
	)
      )

    (define (POVShoot . args)
      (begin
        (let* ([f (open-output-file (string-append "/tmp/" (getenv "USER") ".ini") 'replace)]
               [os (current-output-port)])
          (current-output-port f)
          (tell/apply this POVProof args)
          (current-output-port os)
          (close-output-port f)
          )))
    
    (define (POVProof . args)
      (POVDescribe)
      )
    
    (define (POVDescribe)
	; background is written as part of scene file
      (if environment (tell environment (POVDescribe)))
      (if (getWidth) (printf "Width=~a\n" (getWidth)))
      (if (getHeight) (printf "Height=~a\n" (getHeight)))
      (if (getQuality) (printf "Quality=~a\n" (getQuality)))
      (if (and (getAntiAlias)
	     (not (= (getAntiAlias) 0))) (printf "Antialias=on\nAntialias_Threshold=~a\n" (* 3 (- 1 (getAntiAlias)))))
      )

    (begin
      (super-new)
      (tell this 
        (description "A Mead Image.")
        (dimensions 640.0 480.0)
        (quality 8)
        (antiAlias 0.5)
	(viewResult #t)
	(keep #f)
	(debug #f)
	(background '(1 1 1))
	(fogColor '(1 1 1))
	(fogAttenuation 100)
        )
        )

  )
  )

(define Camera
  (class MeadObject
    (public background ; deprecated after 5/07
            pos getPos
            coi getCoi
            up getUp
            angle getAngle
            subject getSubject shoot proof film watch
            renderer getRenderer
            direction getDirection
            image getImage
	    type perspective orthographic getType
	    buildMovie)
    (inherit attr getAttr  description)
    (define (type t)
      (attr 'type t))
    (define (perspective)
      (attr 'type 'perspective))
    (define (orthographic)
      (attr 'type 'orthographic))
    (define (getType)
      (getAttr 'type))
    (define (pos p)
      (attr 'pos 
            (if (is-a? p Transform)
                (tell1 p mapPoint (getAttr 'pos))
                p)
            ))
    (define (getPos)
      (getAttr 'pos))
    
    (define (coi p)
      (attr 'coi 
            (if (is-a? p Transform)
                (tell1 p mapPoint (getAttr 'coi))
                p)))
    
    (define (getCoi)
      (getAttr 'coi))
    (define (background color)
      (printf "PLEASE: Set background color with (tell image (background color))\n")
      (if (getImage)
	  (tell (getImage) (background color))
	  )
      )
    (define (direction v)
      (let ([pos (getPos)])
        (coi (list (+ (car pos) (car v)) (+ (cadr pos) (cadr v)) (+ (caddr pos) (caddr v))))))
    (define (getDirection)
      (let ([p (getPos)]
            [c (getCoi)])
        (list (- (car c) (car p)) (- (cadr c) (cadr p)) (- (caddr c) (caddr p)))))
    
    (define (up p)
      (attr 'up p)
      )
    (define (getUp)
      (getAttr 'up))
    
    (define (angle a)
      (attr 'angle a)
      )
    (define (getAngle)
      (getAttr 'angle)
      )
    (define (subject theScene)
      (attr 'subject theScene))
    (define (getSubject)
      (eval (getAttr 'subject)))
    (define (renderer r)
      (attr 'renderer r))
    (define (getRenderer)
      (getAttr 'renderer))
    (define (image i)
      (attr 'image i)
      )
    (define (getImage)
      (getAttr 'image)
      )
    
    (define (shoot . args)
      (let ([r (getAttr 'renderer)])
        (cond (else (tell/apply this POVShoot args)))))
    (define (proof . args)
      (let ([r (getAttr 'renderer)])
        (cond (else (tell/apply this POVProof args)))))

    (define (film n setupFun . params)
      (map
       (lambda (t)
	 (let* ([p (map (lambda (param)
			  (if (list? param)
			      (apply blend (cons t param))
			      param))
			params)])
	   (apply setupFun p)
	   (shoot))
	   )
       (morph n 0 1))
      void)

    (define (watch n setupFun . params)
      (let* ([i (getImage)])
	(map
	 (lambda (t)
	   (let* ([p (map (lambda (param)
			    (if (list? param)
				(apply blend (cons t param))
				param))
			  params)])
	     (apply setupFun p)
	     (tell i (advanceFrame)))
	   )
	 (morph n 0 1))
	void
	))

    (define/public (POVShoot . args)
      ;; this method writes the output to a POV temporary file,
      ;; and the executes the POV raytracer on the temporary file.
      ;; the result is stored in gif file
      (begin
        (let* ([f (open-output-file 
		     (string-append "/tmp/" (getenv "USER") ".pov") 'replace)]
               [os (current-output-port)]
	       [i (getImage)]
	       [fnsw (tell i (getFileNameSwitches))]
	       )
          (tell/apply i POVShoot args)
          (current-output-port f)
          (tell/apply this POVProof args)
          (current-output-port os)
          (close-output-port f)
          (system (string-append "/usr/local/bin/mead-pov" fnsw))
	  )
	)
      )

    (define/public (POVProof . args)
      (if (getImage)
	  (let* ([i (getImage)]
		 [b (tell i (getBackground))]
		 [f (tell i (getFog))]
		 [fc (tell i (getFogColor))]
		 [fa (/ (tell i (getFogAttenuation)) (log 2))])
	    (if b
		(begin
		  (printf "background {")
		  (tell pov (writeColor b))
		  (printf "}\n"))
		)
	    (if (and f (= f 1))
		(begin
		  (printf "fog { fog_type ~a " f)
		  (if fc (tell pov (writeColor fc)))
		  (printf " distance ~a " fa)
		  (printf "}~n")
		  )
		)
	    )
	  )
      (let ([theContext (new Context)])
        (if (and (not (null? args)) (procedure? (car args)))
            (tell1 theContext selectionPredicate (car args)))
        (tell1 theContext maxDepth 3)
        (tell1 (getSubject) POVDescribe theContext)
        (tell1 this POVDescribe theContext))
      )
    
    (define (buildMovie)
      ;; this method writes the output to a POV temporary file,
      ;; and the executes the POV raytracer on the temporary file.
      ;; the result is stored in gif file
      (let* ([i (getImage)]
	     [pn (tell i (getFilePath))]
	     )
	(system (string-append "/usr/local/bin/mead-buildMovie --project \"" pn "\" --framecount " (format "~a" (tell i (getFrameNumber)))))
	)
      )

    (define/public (POVDescribe context)
      (let* ([ar (tell (getImage) (getAspectRatio))]
	     [ul (vectorLength (getUp))]
	     [right (vectorScale (normalize (crossProduct (getUp) (getDirection))) (* ul ar))]
	     )
      (printf "camera {")
      (printf " ~a " (getType))
      (printf " right ~a*x" (tell (getImage) (getAspectRatio)))
      (printf " up y ")

      (printf " sky ")
      (tell pov (writePoint (getUp)))
      (printf " location ")
      (tell pov (writePoint (getPos)))
      (printf " look_at ")
      (tell pov (writePoint (getCoi)))
      (printf " angle ~a " (getAngle))
      (printf "}\n")
      )
      )
    
    (begin
      (super-new)
      (tell this 
        (description "A simple camera.")
	(perspective)
        (image (new Image))
        (coi '(0 0 0))
        (pos '(0 100 -200))
        (up '(0 1 0))
        (angle 53)
	(subject 'scene)
        (renderer 'POV)
        ))
    )
  )

(define StereoCamera
  (class Camera
    (inherit pos getPos getCoi getImage POVProof attr getAttr up getUp direction getDirection)
    (public separation getSeparation getLeftPos getRightPos)
    (define (separation d)
      (attr 'separation d))
    (define (getSeparation)
      (getAttr 'separation))
    (define (getLeftPos)
      (let* ([s (getSeparation)]
            [u (getUp)]
            [p (getPos)]
            [d (getDirection)]
            [c (normalize (crossProduct d u))]
            [halfS (/ s 2)])
        (list (+ (car p) (* halfS (car c))) (+ (cadr p) (* halfS (cadr c))) (+ (caddr p) (* halfS (caddr c))))))
    (define (getRightPos)
      (let* ([s (getSeparation)]
            [u (getUp)]
            [p (getPos)]
            [d (getDirection)]
            [c (normalize (crossProduct u d))]
            [halfS (/ s 2)])
        (list (+ (car p) (* halfS (car c))) (+ (cadr p) (* halfS (cadr c))) (+ (caddr p) (* halfS (caddr c))))))
   
        
    (define/override (POVShoot . args)
      ;; this method writes the output to a POV temporary file,
      ;; and the executes the POV raytracer on the temporary file.
      ;; the result is stored in gif file
      (begin
        (let* ([lf (open-output-file (string-append "/tmp/" (getenv "USER") "-left.pov") 'replace)]
               [rf (open-output-file (string-append "/tmp/" (getenv "USER") "-right.pov") 'replace)]
               [os (current-output-port)]
               [nosePos (getPos)]
               [leftPos (getLeftPos)]
               [rightPos (getRightPos)]
	       [i (getImage)]
	       [fnsw (tell i (getFileNameSwitches))]
	       [vsw (if (tell i (getViewResult)) "" " --batch")])
          (tell/apply i POVShoot args)
          (current-output-port lf)
          (pos leftPos)
          (tell/apply this POVProof args)
          (current-output-port rf)
          (close-output-port lf)
          (pos rightPos)
          (tell/apply this POVProof args)
          (current-output-port os)
          (close-output-port rf)
          (pos nosePos)
          (system (string-append "/usr/local/bin/mead-pov --stereo" fnsw vsw)))))
    
    (begin
      (super-new)
      (tell this 
        (description "A stereo camera.")
	; the following angle determines by eye separation; experimental
        (separation 10)
        ))
    )
  )

(define IsometricCamera
  (class Camera
    (inherit pos getPos getImage POVProof attr getAttr up getUp direction getDirection)
    (define (getRightPos)
      (tell (yRot -90) (mapPoint (getPos))))
    (define (getTopPos)
      (tell (xRot 90) (mapPoint (getPos))))
    (define (getOrthoPos)
      (tell (xform (yRot -30) (xRot 30)) (mapPoint (getPos))))

    (define/override (POVShoot . args)
      ;; this method writes the output to a POV temporary file,
      ;; and the executes the POV raytracer on the temporary file.
      ;; the result is stored in gif file
      (begin
        (let* ([of (open-output-file (string-append "/tmp/" (getenv "USER") "-ortho.pov") 'replace)]
		[ff (open-output-file (string-append "/tmp/" (getenv "USER") "-front.pov") 'replace)]
               [rf (open-output-file (string-append "/tmp/" (getenv "USER") "-right.pov") 'replace)]
               [tf (open-output-file (string-append "/tmp/" (getenv "USER") "-top.pov") 'replace)]
               [os (current-output-port)]
               [frontPos (getPos)]
               [orthoPos (getOrthoPos)]
               [rightPos (getRightPos)]
               [topPos (getTopPos)]
	       [i (getImage)]
	       [fnsw (tell i (getFileNameSwitches))]
	       [vsw (if (tell i (getViewResult)) "" " --batch")])
          (tell/apply i POVShoot args)
          (current-output-port of)
          (pos orthoPos)
          (tell/apply this POVProof args)
          (current-output-port ff)
	  (close-output-port of)
          (pos frontPos)
          (tell/apply this POVProof args)
          (current-output-port rf)
	  (close-output-port ff)
          (pos rightPos)
          (tell/apply this POVProof args)
          (current-output-port tf)
          (close-output-port rf)
          (pos topPos)
          (tell/apply this POVProof args)
          (current-output-port os)
          (close-output-port tf)
          (pos frontPos)
          (system (string-append "/usr/local/bin/mead-pov --isometric" fnsw vsw)))))
    
    (begin
      (super-new)
      (tell this 
        (description "A isographic camera.")
	(orthographic)
        (pos '(0 0 -500))
	(coi '(0 0 0))
        ))
    )
  )

)
