(module Lights mzscheme
  (require (lib "Syntax.ss" "Mead")
           (lib "Math.ss" "Mead")
           (lib "Object.ss" "Mead")
           (lib "Cameras.ss" "Mead")
           ) ;; this incorporates Materials, as well.
  (provide Light Spotlight LightArray)

(define Light
  (class Transformable
    (inherit selected attr getAttr getXform description
             translate xRot yRot zRot scale
             xyMirror xzMirror yzMirror)
    
    (define/public (pos p) (attr 'pos p))
    (define/public (getPos) (getAttr 'pos))
    
    (define/public (color c)
      (attr 'color c))
    
    (define/public (getColor)
      (getAttr 'color))

    (define/public (shape o)
      (attr 'shape o)
      )
    (define/public (getShape)
      (getAttr 'shape))

    (define/public (fill)
      (shadows #f))

    (define/public (shadows b)
      (attr 'shadows b))
    (define/public (getShadows)
      (getAttr 'shadows))
    
    (define/public (intensity i)
      (attr 'intensity i))
    (define/public (getIntensity)
      (getAttr 'intensity))
    (define/public (POVDescribeTransform context)
      (super POVDescribe context)
      )
    (define/override (POVDescribe context)
      (if (selected context)
          (begin
            (tell context (pushTransform (getXform)))
            (printf "light_source { ")
            (tell pov (writePoint (tell context (mapPoint (getPos)))))
            (printf "\n")
            (let* ([c (getColor)]
                   [r (car c)]
                   [g (cadr c)]
                   [b (caddr c)]
                   [i (getIntensity)])
              (tell pov (writeColor (list (* i r) (* i g) (* i b))))
	      (if (not (getShadows)) (printf " shadowless")))
            (printf "\n")
	    (if (getShape)
		(begin
		  (printf " looks_like { ")
		  (tell1 (getShape) POVDescribe context)
		  (printf " }\n")
		  )
		)
            ;(super POVDescribe context)
            (printf "}\n")
            (tell1 context popTransform)
            )
          )
      )
    
    (begin
      (super-new)
      (tell this 
        (description "A light.")
        (pos '(0 0 0))
        (color '(1 1 1))
	(shadows #t)
        (intensity 0.5))
      )
    )
  )

(define Spotlight
  (class Light
    (inherit pos getPos color getColor intensity getIntensity
             selected attr getAttr getXform description
	     fill shadows getShadows
             translate xRot yRot zRot scale
             xyMirror xzMirror yzMirror
             POVDescribeTransform)
    
    (define/public (coi p)
      (attr 'coi p))
    
    (define/public (getCoi)
      (getAttr 'coi))
    
    (define/public (radius a)
      (attr 'radius a)
     )
    (define/public (getRadius)
      (getAttr 'radius))
    
    (define/public (falloff da)
      (attr 'falloff da)
      )
    (define/public (getFalloff)
      (getAttr 'falloff))
    
    (define/public (tightness t)
      (attr 'tightness 0))
    (define/public (getTightness)
      (getAttr 'tightness))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (begin
            (tell1 context pushTransform (getXform))
            (printf "light_source { ")
            (tell1 pov writePoint (tell1 context mapPoint (getPos)))
            (printf "\n")
            (let* ([c (getColor)]
                   [r (car c)]
                   [g (cadr c)]
                   [b (caddr c)]
                   [i (getIntensity)])
              (tell1 pov writeColor (list (* i r) (* i g) (* i b)))
	      (if (not (getShadows)) (printf " shadowless"))
	      )
            (printf " spotlight\n")
            (printf " point_at ")
            (tell1 pov writePoint (getCoi))
            (printf "\n")
              (printf "radius ~a falloff ~a tightness ~a\n"
                    (getRadius)
                    (+ (getRadius) (getFalloff))
                    (getTightness))
              
            ;(POVDescribeTransform context)
            (printf "}\n")
            (tell1 context popTransform)
            )
          )
      )
    
    (begin
      (super-new)
      (radius 10)
      (falloff 5)
      (tightness 0)
      (coi '(0 0 0)))
    )
  )

(define LightArray
  (class Light
    (inherit pos getPos color getColor intensity getIntensity
             selected attr getAttr getXform description
	     shadows fill getShadows
             translate xRot yRot zRot scale
             xyMirror xzMirror yzMirror
             POVDescribeTransform)
    
    (define/public (cols x)
      (attr 'cols x))
    (define/public (getCols)
      (getAttr 'cols))

    (define/public (rows z)
      (attr 'rows z))
    (define/public (getRows)
      (getAttr 'rows))

    (define/public (width v)
      (attr 'width v))
    (define/public (getWidth)
      (getAttr 'width))
    (define/public (height h)
      (attr 'height h))
    (define/public (getHeight)
      (getAttr 'height))
    
    (define/override (POVDescribe context)
       (if (selected context)
          (begin
            (tell1 context pushTransform (getXform))
            (printf "light_source { ")
            (tell1 pov writePoint (tell1 context mapPoint (getPos)))
            (printf "\n")
            (let* ([c (getColor)]
                   [r (car c)]
                   [g (cadr c)]
                   [b (caddr c)]
                   [i (getIntensity)])
              (tell1 pov writeColor (list (* i r) (* i g) (* i b)))
	      (if (not (getShadows)) (printf " shadowless")))
            (printf " area_light\n")
            (let* ([w (getWidth)]
                   [h (getHeight)]
                   [wd (tell1 context mapDirection w)]
                   [hd (tell1 context mapDirection h)])
              (tell1 pov writePoint wd)
              (printf ",")
              (tell1 pov writePoint hd)
              (printf ", ~a, ~a\n" (getCols) (getRows) )
     )       
            ;(POVDescribeTransform context)
            (printf "}\n")
            (tell1 context popTransform)
            )
          )
      )
    
    (begin
      (super-new)
      (cols 2)
      (rows 2)
      (width '(100 0 0))
      (height '(0 0 100))
      (pos '(0 0 0))
      )
    )
  )
)
