(module Polyhedra mzscheme
  (require (lib "Syntax.ss" "Mead")
           (lib "Math.ss" "Mead")
           (lib "Object.ss" "Mead")
           (lib "Meshes.ss" "Mead"))
  (provide Tetrahedron Hexahedron Octahedron Dodecahedron Icosahedron)
;;
;; Definition of a UV-mapped cube.  Faces span the uv-coordinates between (0,0) and (1,1).
;; (The internal cube uses a UV-mapping that wraps the cube with an image with a single image.)
(define Hexahedron
  (class Mesh
	 (inherit addUVPoly)
	 (begin
	   (super-new)
	   (let ([face (polyXform (2to3d '((-50 -50) (50 -50) (50 50) (-50 50))) (translate 0 0 -50))]
		 [uvFace '((0 0) (1 0) (1 1) (0 1))])
	     (addUVPoly face uvFace)
	     (addUVPoly (polyXform face (xRot 90)) uvFace)
	     (addUVPoly (polyXform face (xRot -90)) uvFace)
	     (addUVPoly (polyXform face (xRot -180)) uvFace)
	     (addUVPoly (polyXform face (yRot 90)) uvFace)
	     (addUVPoly (polyXform face (yRot -90)) uvFace)
	     )
	   )
	 )
  )

;; A tetrahedron.
(define Tetrahedron
  (class Mesh
	 (inherit addUVPoly)
	 (begin
	   (super-new)
	   (let ([uvFace (list '(0 0) '(1 0) (list (cos (/ pi 3)) (sin (/ pi 3))))]
		 [a '(50 50 50)]
		 [b '(-50 -50 50)]
		 [c '(-50 50 -50)]
		 [d '(50 -50 -50)])
	     (addUVPoly (list a b c) uvFace)
	     (addUVPoly (list a b d) uvFace)
	     (addUVPoly (list a c d) uvFace)
	     (addUVPoly (list b c d) uvFace)
	     )
	   )
	 )
  )

;; An octahedron
(define Octahedron
  (class Mesh
	 (inherit addUVPoly)
	 (begin
	   (super-new)
	   (let ([uvFace (list '(0 0) '(1 0) (list (cos (/ pi 3)) (sin (/ pi 3))))]
		 [up '(0 50 0)]
		 [down '(0 -50 0)]
		 [west '(-50 0 0)]
		 [east '(50 0 0)]
		 [north '(0 0 50)]
		 [south '(0 0 -50)])
	     (addUVPoly (list west south up) uvFace)
	     (addUVPoly (list south east up) uvFace)
	     (addUVPoly (list east north up) uvFace)
	     (addUVPoly (list north west up) uvFace)
	     (addUVPoly (list south west down) uvFace)
	     (addUVPoly (list west north down) uvFace)
	     (addUVPoly (list north east down) uvFace)
	     (addUVPoly (list east south down) uvFace)
	     )
	   )
	 )
  )


;; A dodecahedron.
(define Dodecahedron
  (class Mesh
	 (inherit addUVPoly)
	 (begin
	   (super-new)
	   (let* ([s 28.86738474]
		  [b (* s (/ (+ 1 (sqrt 5)) 2))]
		  [a (* s (/ 2 (+ 1 (sqrt 5))))]
		  [p0 (list s s s)]
		  [p1 (list (- s) s s)]
		  [p2 (list s (- s) s)]
		  [p3 (list s s (- s))]
		  [p4 (list (- s) (- s) s)]
		  [p5 (list (- s) s (- s))]
		  [p6 (list s (- s) (- s))]
		  [p7 (list (- s) (- s) (- s))]
		  [p8 (list 0 a b)]
		  [p9 (list b 0 a)]
		  [p10 (list a b 0)]
		  [p11 (list 0 (- a) b)]
		  [p12 (list b 0 (- a))]
		  [p13 (list (- a) b 0)]
		  [p14 (list 0 a (- b))]
		  [p15 (list (- b) 0 a)]
		  [p16 (list a (- b) 0)]
		  [p17 (list 0 (- a) (- b))]
		  [p18 (list (- b) 0 (- a))]
		  [p19 (list (- a) (- b) 0)]
		  [uvFace '((0.34549 0.0)
			    (0.9045069943749474 0.18163563200134025)
			    (0.9045069943749474 0.7694208842938134)
			    (0.34549 0.9510565162951536)
			    (0 0.4755282581475769))])
	     (addUVPoly (list p0 p8 p1 p13 p10) uvFace)
	     (addUVPoly (list p0 p9 p2 p11 p8) uvFace)
	     (addUVPoly (list p0 p10 p3 p12 p9) uvFace)
	     (addUVPoly (list p4 p15 p1 p8 p11) uvFace)
	     (addUVPoly (list p4 p11 p2 p16 p19) uvFace)
	     (addUVPoly (list p4 p19 p7 p18 p15) uvFace)
	     (addUVPoly (list p5 p13 p1 p15 p18) uvFace)
	     (addUVPoly (list p5 p14 p3 p10 p13) uvFace)
	     (addUVPoly (list p5 p18 p7 p17 p14) uvFace)
	     (addUVPoly (list p6 p16 p2 p9 p12) uvFace)
	     (addUVPoly (list p6 p12 p3 p14 p17) uvFace)
	     (addUVPoly (list p6 p17 p7 p19 p16) uvFace)
	     )
	   )
	 )
  )

;; An icosohedron
(define Icosahedron
  (class Mesh
	 (inherit addUVPoly)
	 (begin
	   (super-new)
	   (let* ([a 25.1] ;24.97027456
		  [b (* a (/ (+ 1 (sqrt 5)) 2))]
		  [p0 (list 0 a b)]
		  [p1 (list b 0 a)]
		  [p2 (list a b 0)]
		  [p3 (list 0 (- a) b)]
		  [p4 (list b 0 (- a))]
		  [p5 (list (- a) b 0)]
		  [p6 (list 0 a (- b))]
		  [p7 (list (- b) 0 a)]
		  [p8 (list a (- b) 0)]
		  [p9 (list 0 (- a) (- b))]
		  [p10 (list (- b) 0 (- a))]
		  [p11 (list (- a) (- b) 0)]
		  [uvFace (list '(0 0) '(1 0) (list (cos (/ pi 3)) (sin (/ pi 3))))]
		  )
	     (addUVPoly (list p1 p2 p0) uvFace)
	     (addUVPoly (list p2 p5 p0) uvFace)
	     (addUVPoly (list p5 p7 p0) uvFace)
	     (addUVPoly (list p7 p3 p0) uvFace)
	     (addUVPoly (list p3 p1 p0) uvFace)
	     (addUVPoly (list p2 p1 p4) uvFace)
	     (addUVPoly (list p5 p2 p6) uvFace)
	     (addUVPoly (list p7 p5 p10) uvFace)
	     (addUVPoly (list p3 p7 p11) uvFace)
	     (addUVPoly (list p1 p3 p8) uvFace)
	     (addUVPoly (list p8 p4 p1) uvFace)
	     (addUVPoly (list p4 p6 p2) uvFace)
	     (addUVPoly (list p6 p10 p5) uvFace)
	     (addUVPoly (list p10 p11 p7) uvFace)
	     (addUVPoly (list p11 p8 p3) uvFace)
	     (addUVPoly (list p4 p8 p9) uvFace)
	     (addUVPoly (list p8 p11 p9) uvFace)
	     (addUVPoly (list p11 p10 p9) uvFace)
	     (addUVPoly (list p10 p6 p9) uvFace)
	     (addUVPoly (list p6 p4 p9) uvFace)
	     )
	   )
	 )
  )
)
