(module SOR mzscheme
  (require (lib "Syntax.ss" "Mead")
           (lib "Math.ss" "Mead")
           (lib "Object.ss" "Mead")
           )
  (provide Lathe Prism)
(define Lathe
  (class Primitive
    (inherit selected attr getAttr getXform)
    
    (define/public (profile l)
      (attr 'profile l))
    (define/public (getProfile)
      (getAttr 'profile))
    
    (define/public (linear)
      (attr 'type 'linear))
    (define/public (bezier)
      (attr 'type 'bezier))
    (define/public (cubic)
      (attr 'type 'cubic))
    (define/public (sturm)
      (attr 'sturm #t))
    (define/public (getSturm)
      (getAttr 'sturm))
    
    (define/public (getType)
      (getAttr 'type))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([p (getProfile)]
                 [t (getType)])
            (printf "lathe { ")
                        (cond 
                          ([eq? t 'cubic] (printf " cubic_spline "))
                          ([eq? t 'bezier] (printf " bezier_spline "))
                  (else (printf " linear_spline ")))

            (printf "~a\n" (length p))
            (do ([p p (cdr p)])
              ([null? p])
              (printf ",<~a,~a>" (caar p) (cadar p))
              )
            (printf "\n")
            (super POVDescribe context)
            (if (getSturm) (printf " sturm "))
            (printf "}\n"))))
    (begin
      (super-new)
      (linear)
      )))

(define Prism
  (class Primitive
    (inherit selected attr getAttr getXform xRot)
    
    (define/public (profile l)
      (attr 'profile l))
    (define/public (getProfile)
      (getAttr 'profile))
    (define/public (linear)
      (attr 'type 'linear))
    (define/public (bezier)
      (attr 'type 'bezier))
    (define/public (cubic)
      (attr 'type 'cubic))
    (define/public (sturm)
      (attr 'sturm #t))
    (define/public (getSturm)
      (getAttr 'sturm))
    
    (define/public (getType)
      (getAttr 'type))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([p (getProfile)]
                 [t (getType)]
	         )
            (printf "prism { ")
                        (cond 
                          ([eq? t 'cubic] (printf " cubic_spline "))
                          ([eq? t 'bezier] (printf " bezier_spline "))
                  (else (printf " linear_spline ")))
            (printf "-50 50 ~a\n" (length p))
            (do ([p p (cdr p)])
              ([null? p])
              (printf ",<~a,~a>" (caar p) (cadar p))
              )
            (printf "\n")
            (super POVDescribe context)
            (if (getSturm) (printf " sturm "))
            (printf "}\n"))))
    (begin
      (super-new)
      (linear)
      (xRot -90)
      )))
)
