(module SimpleObjects mzscheme
  (require (lib "Syntax.ss" "Mead")
           (lib "Object.ss" "Mead")
          )
  (provide Cube Plane Sphere Torus Superellipsoid Cone Cylinder)
(define Cube
  (class Primitive
    (inherit selected description attr getAttr getXform material getMaterial)
    (define/private (shape dx dy dz)
      (attr 'shape (list dx dy dz))
      )
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([s (getAttr 'shape)]
                 [x (car s)]
                 [y (cadr s)]
                 [z (caddr s)]
                 [t (getXform)])
            (printf "box { ")
            (printPoint (list (- (/ x 2)) (- (/ y 2)) (- (/ z 2))))
            (printPoint (list     (/ x 2)     (/ y 2)    (/ z 2)))
            (printf "\n")
            (super POVDescribe context)
            (printf "}\n")
            )
          )
      )
    (begin
      (super-new)
      (description "A primitive box.")
      (shape 100 100 100)
      )
    )
  )

(define Plane
  (class Primitive
    (inherit selected description attr getAttr getXform getMaterial)
    (define/public (normal x y z)
      (attr 'normal (list x y z)))
    (define/public (getNormal)
      (getAttr 'normal))
    (define/public (offset d)
      (attr 'offset d))
    (define/public (getOffset)
      (getAttr 'offset))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([n (getNormal)]
                 [o (getOffset)]
                 [t (getXform)]
                 [m (getMaterial)])
            (printf "plane { ")
            (printPoint n)
            (printf ",~a\n" o)
            (super POVDescribe context)
            (printf "}\n")
            )
          )
      )
    (begin
      (super-new)
      (normal 0 1 0)
      (offset 0)
      (description "A primitive plane."))
    )
  )

(define Sphere
  (class Primitive
    (inherit selected description attr getAttr getXform)
    (define/public (center p)
      (attr 'center p)
      )
    (define/public (getCenter) (getAttr 'center))
    (define/public (radius r) (attr 'radius r))
    (define/public (getRadius) (getAttr 'radius))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([c (getCenter)]
                 [r (getRadius)])
            (printf "sphere { ")
            (printPoint c)
            (printf ",~a\n" r)
            (super POVDescribe context)
            (printf "}\n")
            )
          )
      )
    
    (begin
      (super-new)
      (description "A primitive sphere.")
      (center '(0 0 0))
      (radius 50)
      )
    )
  )

(define Superellipsoid
  (class Primitive
    (inherit selected description attr getAttr getXform material getMaterial)
    (define/public (roundness a b)
      (attr 'roundness (list a b))
      )
    (define/public (getRoundness)
      (getAttr 'roundness))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([rounds (getRoundness)]
                 [e (car rounds)]
                 [n (cadr rounds)]
                 [m (getMaterial)]
                 [t (getXform)])
            (printf "superellipsoid { <~a,~a>\n" e n)
            (super POVDescribe context)
            (printf "}\n")
            )
          )
      )
    
    (begin
      (super-new)
      (description "A primitive superellipsoid.")
      (tell1 this scale 50 50 50)
      (roundness 1 1)
      )
    )
  )

(define Torus
  (class Primitive
    (inherit selected description attr getAttr getXform material getMaterial
             translate xRot yRot zRot scale
             xyMirror xzMirror yzMirror)
    (define/public (sturmian)
      (attr 'sturm #t))
    (define/public (getSturmian)
      (getAttr 'sturmian))
    
    (define/public (minor radius)
      (attr 'minor radius))
    (define/public (getMinor)
      (getAttr 'minor))
    (define/public (major radius)
      (attr 'major radius))
    (define/public (getMajor)
      (getAttr 'major))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([min (getMinor)]
                 [maj (getMajor)]
                 [t (getXform)]
                 [m (getMaterial)])
            (printf "torus {")
            (if (getSturmian) (printf "sturm "))
            (printf "~a,~a\n" maj min)
            (super POVDescribe context)
            (printf "}\n")
            )
          )
      )
    
    (begin
      (super-new)
      (major 75)
      (minor 25)
      )))


(define Cone
  (class Primitive
    (inherit selected description attr getAttr getXform material getMaterial)
    (define/public (topCenter p)
      (attr 'topCenter p))
    (define/public (getTopCenter)
      (getAttr 'topCenter))
    (define/public (topRadius r)
      (attr 'topRadius r))
    (define/public (getTopRadius)
      (getAttr 'topRadius))
    (define/public (bottomCenter p)
      (attr 'bottomCenter p))
    (define/public (getBottomCenter)
      (getAttr 'bottomCenter))
    (define/public (bottomRadius r)
      (attr 'bottomRadius r))
    (define/public (getBottomRadius)
      (getAttr 'bottomRadius))
    (define/public (capped b)
      (attr 'capped b))
    (define/public (capped?)
      (getAttr 'capped))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([topCenter (getTopCenter)]
                 [bottomCenter (getBottomCenter)]
                 [topRadius (getTopRadius)]
                 [bottomRadius (getBottomRadius)]
                )
            (printf "cone { ")
            (printPoint topCenter)
            (printf ",~a " topRadius)
            (printPoint bottomCenter)
            (printf ",~a\n" bottomRadius)
            (if (not (capped?)) (printf "open\n"))
            (super POVDescribe context)
            (printf "}\n")
            )
          )
      )
    
    (begin
      (super-new)
      (description "A primitive cone.")
      (topCenter '(0 50 0))
      (topRadius 0)
      (bottomCenter '(0 -50 0))
      (bottomRadius 50)
      (capped #t)
      )
    )
  )

(define Cylinder
  (class Primitive
    (inherit selected description attr getAttr getXform material getMaterial)
    (define/public (topCenter p)
      (attr 'topCenter p))
    (define/public (bottomCenter p)
      (attr 'bottomCenter p))
    (define/public (radius r)
      (attr 'radius r))
    
    (define/public (getTopCenter)
      (getAttr 'topCenter))
    (define/public (getBottomCenter)
      (getAttr 'bottomCenter))
    (define/public (getRadius)
      (getAttr 'radius))
    
    
    (define/public (capped b)
      (attr 'capped b))
    (define/public (capped?)
      (getAttr 'capped))
    
    (define/override (POVDescribe context)
      (if (selected context)
          (let* ([top (getTopCenter)]
                 [bottom (getBottomCenter)]
                 [radius (getRadius)]
                 [t (getXform)]
                 [m (getMaterial)])
            (printf "cylinder { ")
            (printPoint top)
            (printPoint bottom)
            (printf ",~a\n" radius)
            (if (not (capped?)) (printf "open\n"))
            (super POVDescribe context)
            (printf "}\n")
            )
          )
      )
    
    (begin
      (super-new)
      (description "A primitive cylinder.")
      (topCenter '(0 50 0))
      (bottomCenter '(0 -50 0))
      (radius 50)
      (capped #t)
      )
    )
  )
)
